"""
Code dealing with product (i.e., fits file) delivery.
"""

#c Copyright 2008-2019, the GAVO project
#c
#c This program is free software, covered by the GNU GPL.  See the
#c COPYING file in the source distribution.

from twisted.web import server

from gavo import base
from gavo.protocols import products
from gavo.web import grend
from gavo.web import weberrors


class ProductRenderer(grend.ServiceBasedPage):
	"""The renderer used for delivering products.

	This will only work with a ProductCore since the resulting
	data set has to contain products.Resources.  Thus, you probably
	will not use this in user RDs.
	"""
	name = "get"
	pathFromSegments = ""

	def render(self, request):
		data = {"accref": 
			products.RAccref.fromRequest(self.pathFromSegments, request)}

		# deferring here and going to all the trouble of running a core
		# is probably overkill; currently, the main thing that'd require
		# handwork is figuring out authentication in parallel with what
		# needs to be done for tar files.  We should do better, indded.
		self.runAsyncWithFormalData(data, request
			).addCallback(self._deliver, request
			).addErrback(weberrors.renderDCErrorPage, request)
		return server.NOT_DONE_YET
	
	def _deliver(self, result, request):
		doPreview = self.queryMeta.ctxArgs.get("preview")
		# SvcResult madness; clean up SvcResult and remove this.
		result = getattr(result, "original", result)
		product = result[0]

		# TODO: figure out a good way to see whether what we've got already is a
		# preview.
		if doPreview and not "Preview" in product.__class__.__name__:
			return products.PreviewCacheManager.getPreviewFor(product
				).addCallback(self._deliverPreview, product, request
				).addErrback(self._deliverPreviewFailure, request)

		else:
			if hasattr(product, "render"):
				return product.render(request)
			else:
				base.ui.notifyWarning("Non-product result in product renderer:"
					" %s"%repr(product))
				from gavo.svcs import streaming
				return streaming.streamOut(lambda w, q: w.write(product), request)

	def _deliverPreview(self, content, product, request):
		previewMime = product.pr["preview_mime"] or "image/jpeg"
		request.setHeader("content-type", str(previewMime))
		request.setHeader("content-length", len(content))
		request.write(content)
		request.finish()
	
	def _deliverPreviewFailure(self, failure, request):
		failure.printTraceback()
		data = b"Not an image (preview generation failed, please report)"
		request.setResponseCode(500)
		request.setHeader("content-type", "text/plain")
		request.setHeader("content-length", str(len(data)))
		request.write(data)
		request.finish()

	def getChild(self, name, request):
		self.pathFromSegments = "/".join(request.popSegments(name))
		return self
